@@ -13,43 +13,50 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-import time
-import smtplib
+import os
 import re
+import smtplib
+from subprocess import Popen, PIPE
+import time
 
 from genshi.builder import tag
 
 from trac import __version__
-from trac.config import BoolOption, IntOption, Option
+from trac.config import BoolOption, ExtensionOption, IntOption, Option
 from trac.core import *
-from trac.util.text import CRLF
-from trac.util.translation import _
+from trac.util.text import CRLF, fix_eol
+from trac.util.translation import _, deactivate, reactivate
 
 MAXHEADERLEN = 76
 EMAIL_LOOKALIKE_PATTERN = (
         # the local part
-        r"[a-zA-Z0-9.'=+_-]+" '@'
+        r"[a-zA-Z0-9.'+_-]+" '@'
         # the domain name part (RFC:1035)
         '(?:[a-zA-Z0-9_-]+\.)+' # labels (but also allow '_')
         '[a-zA-Z](?:[-a-zA-Z\d]*[a-zA-Z\d])?' # TLD
         )
 
+
+class IEmailSender(Interface):
+    """Extension point interface for components that allow sending e-mail."""
+    
+    def send(self, from_addr, recipients, message):
+        """Send message to recipients."""
+
+
 class NotificationSystem(Component):
 
+    email_sender = ExtensionOption('notification', 'email_sender',
+                                   IEmailSender, 'SmtpEmailSender',
+        """Name of the component implementing `IEmailSender`.
+        
+        This component is used by the notification system to send emails.
+        Trac currently provides `SmtpEmailSender` for connecting to an SMTP
+        server, and `SendmailEmailSender` for running a `sendmail`-compatible
+        executable. (''since 0.12'')""")
+
     smtp_enabled = BoolOption('notification', 'smtp_enabled', 'false',
-        """Enable SMTP (email) notification.""")
-
-    smtp_server = Option('notification', 'smtp_server', 'localhost',
-        """SMTP server hostname to use for email notifications.""")
-
-    smtp_port = IntOption('notification', 'smtp_port', 25,
-        """SMTP server port to use for email notification.""")
-
-    smtp_user = Option('notification', 'smtp_user', '',
-        """Username for SMTP server. (''since 0.9'').""")
-
-    smtp_password = Option('notification', 'smtp_password', '',
-        """Password for SMTP server. (''since 0.9'').""")
+        """Enable email notification.""")
 
     smtp_from = Option('notification', 'smtp_from', 'trac@localhost',
         """Sender address to use in notification emails.""")
@@ -69,15 +76,16 @@ class NotificationSystem(Component):
            addresses do not appear publicly (Bcc:). (''since 0.10'').""")
            
     smtp_default_domain = Option('notification', 'smtp_default_domain', '',
-        """Default host/domain to append to address that do not specify one""")
+        """Default host/domain to append to address that do not specify
+           one.""")
         
     ignore_domains = Option('notification', 'ignore_domains', '',
         """Comma-separated list of domains that should not be considered
-           part of email addresses (for usernames with Kerberos domains)""")
+           part of email addresses (for usernames with Kerberos domains).""")
            
     admit_domains = Option('notification', 'admit_domains', '',
         """Comma-separated list of domains that should be considered as
-        valid for email addresses (such as localdomain)""")
+        valid for email addresses (such as localdomain).""")
            
     mime_encoding = Option('notification', 'mime_encoding', 'none',
         """Specifies the MIME encoding scheme for emails.
@@ -90,18 +98,15 @@ class NotificationSystem(Component):
     use_public_cc = BoolOption('notification', 'use_public_cc', 'false',
         """Recipients can see email addresses of other CC'ed recipients.
         
-        If this option is disabled (the default), recipients are put on BCC
-        (''since 0.10'').""")
+        If this option is disabled (the default), recipients are put on BCC.
+        (''since 0.10'')""")
 
     use_short_addr = BoolOption('notification', 'use_short_addr', 'false',
-        """Permit email address without a host/domain (i.e. username only)
+        """Permit email address without a host/domain (i.e. username only).
         
         The SMTP server should accept those addresses, and either append
-        a FQDN or use local delivery (''since 0.10'').""")
+        a FQDN or use local delivery. (''since 0.10'')""")
         
-    use_tls = BoolOption('notification', 'use_tls', 'false',
-        """Use SSL/TLS to send notifications (''since 0.10'').""")
-    
     smtp_subject_prefix = Option('notification', 'smtp_subject_prefix',
                                  '__default__', 
         """Text to prepend to subject line of notification emails. 
@@ -110,6 +115,94 @@ class NotificationSystem(Component):
         If no prefix is desired, then specifying an empty option 
         will disable it. (''since 0.10.1'').""")
 
+    def send_email(self, from_addr, recipients, message):
+        """Send message to recipients via e-mail."""
+        self.email_sender.send(from_addr, recipients, message)
+
+
+class SmtpEmailSender(Component):
+    """E-mail sender connecting to an SMTP server."""
+    
+    implements(IEmailSender)
+    
+    smtp_server = Option('notification', 'smtp_server', 'localhost',
+        """SMTP server hostname to use for email notifications.""")
+
+    smtp_port = IntOption('notification', 'smtp_port', 25,
+        """SMTP server port to use for email notification.""")
+
+    smtp_user = Option('notification', 'smtp_user', '',
+        """Username for SMTP server. (''since 0.9'')""")
+
+    smtp_password = Option('notification', 'smtp_password', '',
+        """Password for SMTP server. (''since 0.9'')""")
+
+    use_tls = BoolOption('notification', 'use_tls', 'false',
+        """Use SSL/TLS to send notifications over SMTP. (''since 0.10'')""")
+    
+    def send(self, from_addr, recipients, message):
+        # Ensure the message complies with RFC2822: use CRLF line endings
+        message = fix_eol(message, CRLF)
+        
+        self.log.info("Sending notification through SMTP at %s:%d to %s"
+                      % (self.smtp_server, self.smtp_port, recipients))
+        server = smtplib.SMTP(self.smtp_server, self.smtp_port)
+        # server.set_debuglevel(True)
+        if self.use_tls:
+            server.ehlo()
+            if not server.esmtp_features.has_key('starttls'):
+                raise TracError(_("TLS enabled but server does not support " \
+                                  "TLS"))
+            server.starttls()
+            server.ehlo()
+        if self.smtp_user:
+            server.login(self.smtp_user.encode('utf-8'),
+                         self.smtp_password.encode('utf-8'))
+        start = time.time()
+        server.sendmail(from_addr, recipients, message)
+        t = time.time() - start
+        if t > 5:
+            self.log.warning('Slow mail submission (%.2f s), '
+                             'check your mail setup' % t)
+        if self.use_tls:
+            # avoid false failure detection when the server closes
+            # the SMTP connection with TLS enabled
+            import socket
+            try:
+                server.quit()
+            except socket.sslerror:
+                pass
+        else:
+            server.quit()
+
+
+class SendmailEmailSender(Component):
+    """E-mail sender using a locally-installed sendmail program."""
+    
+    implements(IEmailSender)
+    
+    sendmail_path = Option('notification', 'sendmail_path', 'sendmail',
+        """Path to the sendmail executable.
+        
+        The sendmail program must accept the `-i` and `-f` options.
+         (''since 0.12'')""")
+
+    def send(self, from_addr, recipients, message):
+        # Use native line endings in message
+        message = fix_eol(message, os.linesep)
+
+        self.log.info("Sending notification through sendmail at %s to %s"
+                      % (self.sendmail_path, recipients))
+        cmdline = [self.sendmail_path, "-i", "-f", from_addr]
+        cmdline.extend(recipients)
+        self.log.debug("Sendmail command line: %s" % cmdline)
+        child = Popen(cmdline, bufsize=-1, stdin=PIPE, stdout=PIPE,
+                      stderr=PIPE)
+        out, err = child.communicate(message)
+        if child.returncode or err:
+            raise Exception("Sendmail failed with (%s, %s), command: '%s'"
+                            % (child.returncode, err.strip(), cmdline))
+
 
 class Notify(object):
     """Generic notification class for Trac.
@@ -163,8 +256,6 @@ class Notify(object):
 class NotifyEmail(Notify):
     """Baseclass for notification by email."""
 
-    smtp_server = 'localhost'
-    smtp_port = 25
     from_email = 'trac+tickets@localhost'
     subject = ''
     template_name = None
@@ -172,7 +263,6 @@ class NotifyEmail(Notify):
     addrsep_re = re.compile(r'[;\s,]+')
 
     def __init__(self, env):
-        global EMAIL_LOOKALIKE_PATTERN
         Notify.__init__(self, env)
 
         addrfmt = EMAIL_LOOKALIKE_PATTERN
@@ -183,9 +273,8 @@ class NotifyEmail(Notify):
                                 admit_domains.replace('.','\.').split(',')])
             addrfmt = r'%s@(?:(?:%s)|%s)' % (addrfmt[:pos], addrfmt[pos+1:], 
                                               domains)
-        self.shortaddr_re = re.compile(r'%s$' % addrfmt)
-        self.longaddr_re = re.compile(r'^\s*(.*)\s+<(%s)>\s*$' % addrfmt);
-        self._use_tls = self.env.config.getbool('notification', 'use_tls')
+        self.shortaddr_re = re.compile(r'\s*(%s)\s*$' % addrfmt)
+        self.longaddr_re = re.compile(r'^\s*(.*)\s+<\s*(%s)\s*>\s*$' % addrfmt)
         self._init_pref_encoding()
         domains = self.env.config.get('notification', 'ignore_domains', '')
         self._ignore_domains = [x.strip() for x in domains.lower().split(',')]
@@ -213,30 +302,26 @@ class NotifyEmail(Notify):
             self._charset.header_encoding = SHORTEST
             self._charset.body_encoding = None
         else:
-            raise TracError(_('Invalid email encoding setting: %s' % pref))
+            raise TracError(_('Invalid email encoding setting: %(pref)s',
+                              pref=pref))
 
     def notify(self, resid, subject):
         self.subject = subject
 
         if not self.config.getbool('notification', 'smtp_enabled'):
             return
-        self.smtp_server = self.config['notification'].get('smtp_server')
-        self.smtp_port = self.config['notification'].getint('smtp_port')
         self.from_email = self.config['notification'].get('smtp_from')
         self.from_name = self.config['notification'].get('smtp_from_name')
         self.replyto_email = self.config['notification'].get('smtp_replyto')
         self.from_email = self.from_email or self.replyto_email
         if not self.from_email and not self.replyto_email:
-            raise TracError(tag(tag.p('Unable to send email due to identity '
-                                        'crisis.'),
-                                  tag.p('Neither ', tag.b('notification.from'),
-                                        ' nor ', tag.b('notification.reply_to'),
-                                        'are specified in the configuration.')),
-                              'SMTP Notification Error')
-
-        # Authentication info (optional)
-        self.user_name = self.config['notification'].get('smtp_user')
-        self.password = self.config['notification'].get('smtp_password')
+            raise TracError(tag(
+                    tag.p(_('Unable to send email due to identity crisis.')),
+                    tag.p(_('Neither %(from_)s nor %(reply_to)s are specified '
+                            'in the configuration.',
+                            from_=tag.b('notification.from'),
+                            reply_to=tag.b('notification.reply_to')))),
+                _('SMTP Notification Error'))
 
         Notify.notify(self, resid)
 
@@ -289,7 +374,7 @@ class NotifyEmail(Notify):
 
         mo = self.shortaddr_re.search(address)
         if mo:
-            return mo.group(0)
+            return mo.group(1)
         mo = self.longaddr_re.search(address)
         if mo:
             return mo.group(2)
@@ -299,42 +384,28 @@ class NotifyEmail(Notify):
     def encode_header(self, key, value):
         if isinstance(value, tuple):
             return self.format_header(key, value[0], value[1])
-        if isinstance(value, list):
-            items = []
-            for v in value:
-                items.append(self.encode_header(v))
-            return ',\n\t'.join(items)
         mo = self.longaddr_re.match(value)
         if mo:
             return self.format_header(key, mo.group(1), mo.group(2))
         return self.format_header(key, value)
 
-    def begin_send(self):
-        self.server = smtplib.SMTP(self.smtp_server, self.smtp_port)
-        # self.server.set_debuglevel(True)
-        if self._use_tls:
-            self.server.ehlo()
-            if not self.server.esmtp_features.has_key('starttls'):
-                raise TracError(_("TLS enabled but server does not support " \
-                                  "TLS"))
-            self.server.starttls()
-            self.server.ehlo()
-        if self.user_name:
-            self.server.login(self.user_name.encode('utf-8'),
-                              self.password.encode('utf-8'))
-
     def send(self, torcpts, ccrcpts, mime_headers={}):
         from email.MIMEText import MIMEText
         from email.Utils import formatdate
         stream = self.template.generate(**self.data)
-        body = stream.render('text')
-        projname = self.config.get('project', 'name')
+        # don't translate the e-mail stream
+        t = deactivate()
+        try:
+            body = stream.render('text', encoding='utf-8')
+        finally:
+            reactivate(t)
+        projname = self.env.project_name
         public_cc = self.config.getbool('notification', 'use_public_cc')
         headers = {}
         headers['X-Mailer'] = 'Trac %s, by Edgewall Software' % __version__
         headers['X-Trac-Version'] =  __version__
         headers['X-Trac-Project'] =  projname
-        headers['X-URL'] = self.config.get('project', 'url')
+        headers['X-URL'] = self.env.project_url
         headers['Precedence'] = 'bulk'
         headers['Auto-Submitted'] = 'auto-generated'
         headers['Subject'] = self.subject
@@ -389,29 +460,7 @@ class NotifyEmail(Notify):
         # encoding type to force a new, valid evaluation
         del msg['Content-Transfer-Encoding']
         msg.set_charset(self._charset)
-        self.add_headers(msg, headers);
-        self.add_headers(msg, mime_headers);
-        self.env.log.info("Sending SMTP notification to %s:%d to %s"
-                           % (self.smtp_server, self.smtp_port, recipients))
-        msgtext = msg.as_string()
-        # Ensure the message complies with RFC2822: use CRLF line endings
-        recrlf = re.compile("\r?\n")
-        msgtext = CRLF.join(recrlf.split(msgtext))
-        start = time.time()
-        self.server.sendmail(msg['From'], recipients, msgtext)
-        t = time.time() - start
-        if t > 5:
-            self.env.log.warning('Slow mail submission (%.2f s), '
-                                 'check your mail setup' % t)
-
-    def finish_send(self):
-        if self._use_tls:
-            # avoid false failure detection when the server closes
-            # the SMTP connection with TLS enabled
-            import socket
-            try:
-                self.server.quit()
-            except socket.sslerror:
-                pass
-        else:
-            self.server.quit()
+        self.add_headers(msg, headers)
+        self.add_headers(msg, mime_headers)
+        NotificationSystem(self.env).send_email(self.from_email, recipients,
+                                                msg.as_string())
