@@ -3,7 +3,7 @@
 # Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
-# Copyright (C) 2006 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -447,8 +447,8 @@ class Ticket(object):
                 return
             ts = row[0]
 
-            custom_fields = set(f['name'] for f in self.fields
-                                if f.get('custom'))
+            std_fields = set(f['name'] for f in self.fields
+                             if not f.get('custom'))
 
             # Find modified fields and their previous value
             cursor.execute("""
@@ -475,15 +475,15 @@ class Ticket(object):
                     break
                 else:
                     # No next change, edit ticket field
-                    if field in custom_fields:
+                    if field in std_fields:
+                        cursor.execute("""
+                            UPDATE ticket SET %s=%%s WHERE id=%%s
+                            """ % field, (oldvalue, self.id))
+                    else:
                         cursor.execute("""
                             UPDATE ticket_custom SET value=%s
                             WHERE ticket=%s AND name=%s
                             """, (oldvalue, self.id, field))
-                    else:
-                        cursor.execute("""
-                            UPDATE ticket SET %s=%%s WHERE id=%%s
-                            """ % field, (oldvalue, self.id))
 
             # Delete the change
             cursor.execute("""
