@@ -1,4 +1,3 @@
-from trac.config import Configuration
 from trac.test import EnvironmentStub
 from trac.ticket.roadmap import *
 from trac.core import ComponentManager
@@ -8,11 +7,11 @@ import unittest
 class TicketGroupStatsTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.stats = TicketGroupStats('title', 'unit')
+        self.stats = TicketGroupStats('title', 'units')
 
     def test_init(self):
         self.assertEquals('title', self.stats.title, 'title incorrect')
-        self.assertEquals('unit', self.stats.unit, 'unit incorrect')
+        self.assertEquals('units', self.stats.unit, 'unit incorrect')
         self.assertEquals(0, self.stats.count, 'count not zero')
         self.assertEquals(0, len(self.stats.intervals), 'intervals not empty')
 
@@ -34,7 +33,7 @@ class TicketGroupStatsTestCase(unittest.TestCase):
         self.stats.add_interval('intTitle', 3, {'k1': 'v1'}, 'css', 0)
         self.stats.add_interval('intTitle', 5, {'k1': 'v1'}, 'css', 0)
         self.stats.refresh_calcs()
-        int = self.stats.intervals[1]
+        interval = self.stats.intervals[1]
         self.assertEquals(0, self.stats.done_count, 'count added for no prog')
         self.assertEquals(0, self.stats.done_percent, 'percent incremented')
 
@@ -98,7 +97,7 @@ class DefaultTicketGroupStatsProviderTestCase(unittest.TestCase):
 
     def test_stats(self):
         self.assertEquals(self.stats.title, 'ticket status', 'title incorrect')
-        self.assertEquals(self.stats.unit, 'ticket', 'unit incorrect')
+        self.assertEquals(self.stats.unit, 'tickets', 'unit incorrect')
         self.assertEquals(2, len(self.stats.intervals), 'more than 2 intervals')
 
     def test_closed_interval(self):
