@@ -3,7 +3,7 @@
 # Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
-# Copyright (C) 2005-2006 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -32,7 +32,7 @@ import time
 from urllib import quote, unquote, urlencode
 
 from trac.util.compat import any, md5, sha1, sorted
-from trac.util.text import to_unicode
+from trac.util.text import to_unicode, getpreferredencoding
 
 # -- req/session utils
 
@@ -296,7 +296,7 @@ def copytree(src, dst, symlinks=False, skip=[], overwrite=False):
     def str_path(path):
         if isinstance(path, unicode):
             path = path.encode(sys.getfilesystemencoding() or
-                               locale.getpreferredencoding())
+                               getpreferredencoding())
         return path
 
     def remove_if_overwriting(path):
@@ -715,7 +715,7 @@ class Ranges(object):
 
     """
 
-    RE_STR = r"""\d+(?:[-:]\d+)?(?:,\d+(?:[-:]\d+)?)*"""
+    RE_STR = r"""[0-9]+(?:[-:][0-9]+)?(?:,[0-9]+(?:[-:][0-9]+)?)*"""
     
     def __init__(self, r=None, reorder=False):
         self.pairs = []
