@@ -3,7 +3,7 @@
 # Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
-# Copyright (C) 2005-2006 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -25,7 +25,7 @@ import time
 from datetime import tzinfo, timedelta, datetime, date
 
 from trac.core import TracError
-from trac.util.text import to_unicode
+from trac.util.text import to_unicode, getpreferredencoding
 from trac.util.translation import _, ngettext
 
 # Date/time utilities
@@ -33,22 +33,33 @@ from trac.util.translation import _, ngettext
 # -- conversion
 
 def to_datetime(t, tzinfo=None):
-    """Convert `t` into a `datetime` object, using the following rules:
-    
-     - If `t` is already a `datetime` object, it is simply returned.
-     - If `t` is None, the current time will be used.
-     - If `t` is a number, it is interpreted as a timestamp.
-     
-    If no `tzinfo` is given, the local timezone will be used.
+    """Convert ``t`` into a `datetime` object in the ``tzinfo`` timezone.
+
+    If no ``tzinfo`` is given, the local timezone `localtz` will be used.
+
+    ``t`` is converted using the following rules:
+
+     - If ``t`` is already a `datetime` object,
+       - if it is timezone-"naive", it is localized to ``tzinfo``
+       - if it is already timezone-aware, ``t`` is mapped to the given
+         timezone (`datetime.datetime.astimezone`)
+     - If ``t`` is None, the current time will be used.
+     - If ``t`` is a number, it is interpreted as a timestamp.
 
     Any other input will trigger a `TypeError`.
+
+    All returned datetime instances are timezone aware and normalized.
     """
+    tz = tzinfo or localtz
     if t is None:
-        return datetime.now(tzinfo or localtz)
+        dt = datetime.now(tz)
     elif isinstance(t, datetime):
-        return t
+        if t.tzinfo:
+            dt = t.astimezone(tz)
+        else:
+            dt = tz.localize(t)
     elif isinstance(t, date):
-        return (tzinfo or localtz).localize(datetime(t.year, t.month, t.day))
+        dt = tz.localize(datetime(t.year, t.month, t.day))
     elif isinstance(t, (int, long, float)):
         if not (_min_ts <= t <= _max_ts):
             # Handle microsecond timestamps for 0.11 compatibility
@@ -57,9 +68,12 @@ def to_datetime(t, tzinfo=None):
             # Work around negative fractional times bug in Python 2.4
             # http://bugs.python.org/issue1646728
             frac, integer = math.modf(t)
-            return datetime.fromtimestamp(integer - 1, tzinfo or localtz) \
-                   + timedelta(seconds=frac + 1)
-        return datetime.fromtimestamp(t, tzinfo or localtz)
+            dt = datetime.fromtimestamp(integer - 1, tz) + \
+                    timedelta(seconds=frac + 1)
+        else:
+            dt = datetime.fromtimestamp(t, tz)
+    if dt:
+        return tz.normalize(dt)
     raise TypeError('expecting datetime, int, long, float, or None; got %s' %
                     type(t))
 
@@ -131,7 +145,7 @@ def format_datetime(t=None, format='%x %X', tzinfo=None):
     `tzinfo` will default to the local timezone if left to `None`.
     """
     tz = tzinfo or localtz
-    t = to_datetime(t, tzinfo).astimezone(tz)
+    t = to_datetime(t, tz)
     normalize_Z = False
     if format.lower().startswith('iso8601'):
         if 'date' in format:
@@ -147,7 +161,7 @@ def format_datetime(t=None, format='%x %X', tzinfo=None):
         text = text.replace('+0000', 'Z')
         if not text.endswith('Z'):
             text = text[:-2] + ":" + text[-2:]
-    encoding = locale.getpreferredencoding() or sys.getdefaultencoding()
+    encoding = getpreferredencoding() or sys.getdefaultencoding()
     if sys.platform != 'win32' or sys.version_info[:2] > (2, 3):
         encoding = locale.getlocale(locale.LC_TIME)[1] or encoding
         # Python 2.3 on windows doesn't know about 'XYZ' alias for 'cpXYZ'
@@ -195,7 +209,7 @@ def get_datetime_format_hint():
 
 def http_date(t=None):
     """Format `datetime` object `t` as a rfc822 timestamp"""
-    t = to_datetime(t).astimezone(utc)
+    t = to_datetime(t, utc)
     weekdays = ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun')
     months = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep',
               'Oct', 'Nov', 'Dec')
@@ -239,6 +253,7 @@ def parse_date(text, tzinfo=None, hint='date'):
                                             hours, minutes, seconds),
                                '%Y %m %d %H %M %S ')
             dt = tzinfo.localize(datetime(*tm[0:6]))
+            dt = tzinfo.normalize(dt)
         except ValueError:
             pass
     if dt is None:
@@ -247,6 +262,7 @@ def parse_date(text, tzinfo=None, hint='date'):
             try:
                 tm = time.strptime(text, format)
                 dt = tzinfo.localize(datetime(*tm[0:6]))
+                dt = tzinfo.normalize(dt)
                 break
             except ValueError:
                 continue
@@ -261,7 +277,7 @@ def parse_date(text, tzinfo=None, hint='date'):
     # Make sure we can convert it to a timestamp and back - fromtimestamp()
     # may raise ValueError if larger than platform C localtime() or gmtime()
     try:
-        to_datetime(to_timestamp(dt), tzinfo)
+        datetime.utcfromtimestamp(to_timestamp(dt))
     except ValueError:
         raise TracError(_('The date "%(date)s" is outside valid range. '
                           'Try a date closer to present time.', date=text),
@@ -290,45 +306,55 @@ _time_intervals = dict(
 _TIME_START_RE = re.compile(r'(this|last)\s*'
                             r'(second|minute|hour|day|week|month|year)$')
 _time_starts = dict(
-    second=lambda now: now.replace(microsecond=0),
-    minute=lambda now: now.replace(microsecond=0, second=0),
-    hour=lambda now: now.replace(microsecond=0, second=0, minute=0),
-    day=lambda now: now.replace(microsecond=0, second=0, minute=0, hour=0),
-    week=lambda now: now.replace(microsecond=0, second=0, minute=0, hour=0) \
+    second=lambda now: datetime(now.year, now.month, now.day, now.hour,
+                                now.minute, now.second),
+    minute=lambda now: datetime(now.year, now.month, now.day, now.hour,
+                                now.minute),
+    hour=lambda now: datetime(now.year, now.month, now.day, now.hour),
+    day=lambda now: datetime(now.year, now.month, now.day),
+    week=lambda now: datetime(now.year, now.month, now.day) \
                      - timedelta(days=now.weekday()),
-    month=lambda now: now.replace(microsecond=0, second=0, minute=0, hour=0,
-                                  day=1),
-    year=lambda now: now.replace(microsecond=0, second=0, minute=0, hour=0,
-                                  day=1, month=1),
+    month=lambda now: datetime(now.year, now.month, 1),
+    year=lambda now: datetime(now.year, 1, 1),
 )
 
-def _parse_relative_time(text, tzinfo):
-    now = tzinfo.localize(datetime.now())
+def _parse_relative_time(text, tzinfo, now=None):
+    if now is None:     # now argument for unit tests
+        now = datetime.now(tzinfo)
     if text == 'now':
         return now
+
+    dt = None
     if text == 'today':
-        return now.replace(microsecond=0, second=0, minute=0, hour=0)
-    if text == 'yesterday':
-        return now.replace(microsecond=0, second=0, minute=0, hour=0) \
-               - timedelta(days=1)
-    match = _REL_TIME_RE.match(text)
-    if match:
-        (value, interval) = match.groups()
-        return now - _time_intervals[interval](float(value))
-    match = _TIME_START_RE.match(text)
-    if match:
-        (which, start) = match.groups()
-        dt = _time_starts[start](now)
-        if which == 'last':
-            if start == 'month':
-                if dt.month > 1:
-                    dt = dt.replace(month=dt.month - 1)
+        dt = _time_starts['day'](now)
+    elif text == 'yesterday':
+        dt = _time_starts['day'](now) - timedelta(days=1)
+    if dt is None:
+        match = _REL_TIME_RE.match(text)
+        if match:
+            (value, interval) = match.groups()
+            dt = now - _time_intervals[interval](float(value))
+    if dt is None:
+        match = _TIME_START_RE.match(text)
+        if match:
+            (which, start) = match.groups()
+            dt = _time_starts[start](now)
+            if which == 'last':
+                if start == 'month':
+                    if dt.month > 1:
+                        dt = dt.replace(month=dt.month - 1)
+                    else:
+                        dt = dt.replace(year=dt.year - 1, month=12)
+                elif start == 'year':
+                    dt = dt.replace(year=dt.year - 1)
                 else:
-                    dt = dt.replace(year=dt.year - 1, month=12)
-            else:
-                dt -= _time_intervals[start](1)
-        return dt
-    return None
+                    dt -= _time_intervals[start](1)
+
+    if dt is None:
+        return None
+    if not dt.tzinfo:
+        dt = tzinfo.localize(dt)
+    return tzinfo.normalize(dt)
 
 
 # -- timezone utilities
@@ -366,61 +392,113 @@ class FixedOffset(tzinfo):
         return dt
 
 
-STDOFFSET = timedelta(seconds=-time.timezone)
-if time.daylight:
-    DSTOFFSET = timedelta(seconds=-time.altzone)
-else:
-    DSTOFFSET = STDOFFSET
-
-DSTDIFF = DSTOFFSET - STDOFFSET
-
-
 class LocalTimezone(tzinfo):
     """A 'local' time zone implementation"""
-    
+
+    _std_offset = None
+    _dst_offset = None
+    _dst_diff = None
+    _std_tz = None
+    _dst_tz = None
+
+    @classmethod
+    def _initialize(cls):
+        cls._std_tz = cls(False)
+        cls._std_offset = timedelta(seconds=-time.timezone)
+        if time.daylight:
+            cls._dst_tz = cls(True)
+            cls._dst_offset = timedelta(seconds=-time.altzone)
+        else:
+            cls._dst_tz = cls._std_tz
+            cls._dst_offset = cls._std_offset
+        cls._dst_diff = cls._dst_offset - cls._std_offset
+
+    def __init__(self, is_dst=None):
+        self.is_dst = is_dst
+
     def __str__(self):
-        return self.tzname(datetime.now())
-    
+        offset = self.utcoffset(datetime.now())
+        secs = offset.days * 3600 * 24 + offset.seconds
+        hours, rem = divmod(abs(secs), 3600)
+        return 'UTC%c%02d:%02d' % ('+-'[secs < 0], hours, rem / 60)
+
     def __repr__(self):
-        return '<LocalTimezone "%s" %s "%s" %s>' % (
-            time.tzname[False], STDOFFSET,
-            time.tzname[True], DSTOFFSET)
+        if self.is_dst is None:
+            return '<LocalTimezone "%s" %s "%s" %s>' % \
+                   (time.tzname[False], self._std_offset,
+                    time.tzname[True], self._dst_offset)
+        if self.is_dst:
+            offset = self._dst_offset
+        else:
+            offset = self._std_offset
+        return '<LocalTimezone "%s" %s>' % (time.tzname[self.is_dst], offset)
+
+    def _is_dst(self, dt, is_dst=False):
+        if self.is_dst is not None:
+            return self.is_dst
+
+        tt = (dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second,
+              dt.weekday(), 0)
+        try:
+            std_tt = time.localtime(time.mktime(tt + (0,)))
+            dst_tt = time.localtime(time.mktime(tt + (1,)))
+        except (ValueError, OverflowError):
+            return False
+
+        std_correct = std_tt.tm_isdst == 0
+        dst_correct = dst_tt.tm_isdst == 1
+        if std_correct is dst_correct:
+            if is_dst is None:
+                if std_correct is True:
+                    raise ValueError('Ambiguous time "%s"' % dt)
+                if std_correct is False:
+                    raise ValueError('Non existent time "%s"' % dt)
+            return is_dst
+        if std_correct:
+            return False
+        if dst_correct:
+            return True
 
     def utcoffset(self, dt):
-        if self._isdst(dt):
-            return DSTOFFSET
+        if self._is_dst(dt):
+            return self._dst_offset
         else:
-            return STDOFFSET
+            return self._std_offset
 
     def dst(self, dt):
-        if self._isdst(dt):
-            return DSTDIFF
+        if self._is_dst(dt):
+            return self._dst_diff
         else:
             return _zero
 
     def tzname(self, dt):
-        return time.tzname[self._isdst(dt)]
-
-    def _isdst(self, dt):
-        tt = (dt.year, dt.month, dt.day,
-              dt.hour, dt.minute, dt.second,
-              dt.weekday(), 0, -1)
-        try:
-            stamp = time.mktime(tt)
-            tt = time.localtime(stamp)
-            return tt.tm_isdst > 0
-        except OverflowError:
-            return False
+        return time.tzname[self._is_dst(dt)]
 
     def localize(self, dt, is_dst=False):
         if dt.tzinfo is not None:
             raise ValueError('Not naive datetime (tzinfo is already set)')
-        return dt.replace(tzinfo=self)
+        if self._is_dst(dt, is_dst):
+            tz = self._dst_tz
+        else:
+            tz = self._std_tz
+        return dt.replace(tzinfo=tz)
 
     def normalize(self, dt, is_dst=False):
         if dt.tzinfo is None:
             raise ValueError('Naive time (no tzinfo set)')
-        return dt
+        if dt.tzinfo is localtz: # if not localized, returns without changes
+            return dt
+        return self.fromutc(dt.replace(tzinfo=self) - dt.utcoffset())
+
+    def fromutc(self, dt):
+        if dt.tzinfo is None or dt.tzinfo is not self:
+            raise ValueError('fromutc: dt.tzinfo is not self')
+        tt = time.localtime(to_timestamp(dt.replace(tzinfo=utc)))
+        if tt.tm_isdst > 0:
+            tz = self._dst_tz
+        else:
+            tz = self._std_tz
+        return datetime(microsecond=dt.microsecond, tzinfo=tz, *tt[0:6])
 
 
 utc = FixedOffset(0, 'UTC')
@@ -431,8 +509,14 @@ _zero = timedelta(0)
 _min_ts = -(1 << 31)
 _max_ts = (1 << 31) - 1
 
+LocalTimezone._initialize()
 localtz = LocalTimezone()
 
+STDOFFSET = LocalTimezone._std_offset
+DSTOFFSET = LocalTimezone._dst_offset
+DSTDIFF = LocalTimezone._dst_diff
+
+
 # Use a makeshift timezone implementation if pytz is not available.
 # This implementation only supports fixed offset time zones.
 #
