@@ -294,12 +294,12 @@ try:
         return _js_quote_re.sub(replace, text)
 
 except ImportError:
-    from trac.util.text import javascript_quote
+    from trac.util.text import to_js_string
     
     def to_json(value):
         """Encode `value` to JSON."""
         if isinstance(value, basestring):
-            return '"%s"' % javascript_quote(value)
+            return to_js_string(value)
         elif value is None:
             return 'null'
         elif value is False:
