@@ -2,7 +2,7 @@
 #
 # Copyright (C) 2005-2009 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
-# Copyright (C) 2005-2007 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,7 +14,7 @@
 # history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
-#         Christian Boos <cboos@neuf.fr>
+#         Christian Boos <cboos@edgewall.org>
 
 """Filesystem access to Subversion repositories.
 
@@ -604,6 +604,14 @@ class SubversionRepository(Repository):
 
     def get_changes(self, old_path, old_rev, new_path, new_rev,
                     ignore_ancestry=0):
+         def key(value):
+             return value[1] and value[1].path or value[0].path
+         return iter(sorted(self._get_changes(old_path, old_rev, new_path,
+                                              new_rev, ignore_ancestry),
+                            key=key))
+         
+    def _get_changes(self, old_path, old_rev, new_path, new_rev,
+                     ignore_ancestry):
         old_node = new_node = None
         old_rev = self.normalize_rev(old_rev)
         new_rev = self.normalize_rev(new_rev)
@@ -679,7 +687,6 @@ class SubversionNode(Node):
         self.pool = Pool(pool)
         pool = self.pool()
         self._scoped_path_utf8 = _to_svn(pool, self.scope, path)
-        self._requested_rev = rev
 
         if parent_root:
             self.root = parent_root
@@ -703,9 +710,8 @@ class SubversionNode(Node):
             self.created_path = _path_within_scope(self.scope, cp)
         else:
             self.created_rev, self.created_path = rev, path
-        self.rev = self.created_rev
         # TODO: check node id
-        Node.__init__(self, repos, path, self.rev, _kindmap[node_type])
+        Node.__init__(self, repos, path, rev, _kindmap[node_type])
 
     def get_content(self):
         if self.isdir:
@@ -724,16 +730,15 @@ class SubversionNode(Node):
         entries = fs.dir_entries(self.root, self._scoped_path_utf8, pool())
         for item in entries.keys():
             path = posixpath.join(self.path, _from_svn(item))
-            yield SubversionNode(path, self._requested_rev, self.repos,
-                                 self.pool, self.root)
+            yield SubversionNode(path, self.rev, self.repos, self.pool,
+                                 self.root)
 
     def get_history(self, limit=None):
         newer = None # 'newer' is the previously seen history tuple
         older = None # 'older' is the currently examined history tuple
         pool = Pool(self.pool)
         numrevs = 0
-        for path, rev in self.repos._history(self.path, 1, self._requested_rev,
-                                             pool):
+        for path, rev in self.repos._history(self.path, 1, self.rev, pool):
             path = _path_within_scope(self.scope, path)
             if rev > 0 and path:
                 older = (path, rev, Changeset.ADD)
@@ -757,7 +762,7 @@ class SubversionNode(Node):
             def blame_receiver(line_no, revision, author, date, line, pool):
                 annotations.append(revision)
             try:
-                rev = _svn_rev(self._requested_rev)
+                rev = _svn_rev(self.rev)
                 start = _svn_rev(0)
                 file_url_utf8 = posixpath.join(self.repos.ra_url_utf8,
                                                self._scoped_path_utf8)
@@ -817,7 +822,7 @@ class SubversionNode(Node):
         operation was performed.
         """
         ancestors = []
-        previous = (self._scoped_path_utf8, self._requested_rev, self.root)
+        previous = (self._scoped_path_utf8, self.rev, self.root)
         while previous:
             (previous_path, previous_rev, previous_root) = previous
             previous = None
@@ -904,18 +909,20 @@ class SubversionChangeset(Changeset):
             path = _from_svn(path_utf8)
             base_path = _from_svn(base_path_utf8)
             base_rev = change.base_rev
+            change_action = getattr(change, 'action', None)
 
             # Ensure `base_path` is within the scope
             if not _is_path_within_scope(self.scope, base_path):
                 base_path, base_rev = None, -1
 
             # Determine the action
-            if not path:                # deletion
-                if new_path:
+            if not path and not new_path and self.scope == '/':
+                action = Changeset.EDIT # root property change
+            elif not path or (change_action is not None
+                              and change_action == repos.CHANGE_ACTION_DELETE):
+                if new_path:            # deletion
                     action = Changeset.DELETE
                     deletions[new_path.lstrip('/')] = idx
-                elif self.scope == '/': # root property change
-                    action = Changeset.EDIT
                 else:                   # deletion outside of scope, ignore
                     continue
             elif change.added or not base_path: # add or copy
