@@ -24,6 +24,7 @@ from trac.cache import cached
 from trac.config import BoolOption, ListOption
 from trac.core import *
 from trac.resource import IResourceManager
+from trac.util.compat import all
 from trac.util.translation import _
 from trac.wiki.parser import WikiParser
 
@@ -189,6 +190,15 @@ def parse_args(args, strict=True):
     return largs, kwargs
 
 
+def validate_page_name(pagename):
+    """Utility for validating wiki page name.
+
+    :param pagename: wiki page name to validate
+    """
+    return pagename and \
+           all(part not in ('', '.', '..') for part in pagename.split('/'))
+
+
 class WikiSystem(Component):
     """Wiki system manager."""
 
@@ -280,7 +290,7 @@ class WikiSystem(Component):
     def get_wiki_syntax(self):
         wiki_page_name = (
             r"(?:[%(upper)s](?:[%(lower)s])+/?){2,}" # wiki words
-            r"(?:@\d+)?"                             # optional version
+            r"(?:@[0-9]+)?"                          # optional version
             r"(?:#%(xml)s)?"                         # optional fragment id
             r"(?=:(?:\Z|\s)|[^:\w%(upper)s%(lower)s]|\s|\Z)"
             # what should follow it
