@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright (C) 2005-2009 Edgewall Software
-# Copyright (C) 2005-2006 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,16 +12,16 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Christian Boos <cboos@neuf.fr>
+# Author: Christian Boos <cboos@edgewall.org>
 
 import re
 
-from genshi.builder import Element, tag
+from genshi.builder import Element, Fragment, tag
 
 from trac.core import *
 from trac.mimeview import Context
 from trac.perm import PermissionError
-from trac.util import sorted
+from trac.util.html import find_element
 from trac.util.translation import _
 from trac.web import IRequestHandler
 from trac.wiki.api import IWikiMacroProvider
@@ -29,7 +29,7 @@ from trac.wiki.formatter import extract_link
 
 
 class InterTracDispatcher(Component):
-    """Implements support for InterTrac dispatching."""
+    """InterTrac dispatcher."""
 
     implements(IRequestHandler, IWikiMacroProvider)
 
@@ -44,11 +44,17 @@ class InterTracDispatcher(Component):
 
     def process_request(self, req):
         link = req.args.get('link', '')
-        link_elt = extract_link(self.env, Context.from_request(req), link)
-        if isinstance(link_elt, Element):
-            href = link_elt.attrib.get('href')
-            if href is None: # most probably no permissions to view
+        parts = link.split(':', 1)
+        if len(parts) > 1:
+            resolver, target = parts
+            if target and (target[0] not in '\'"' or target[0] != target[-1]):
+                link = '%s:"%s"' % (resolver, target)
+        link_frag = extract_link(self.env, Context.from_request(req), link)
+        if isinstance(link_frag, (Element, Fragment)):
+            elt = find_element(link_frag, 'href')
+            if elt is None: # most probably no permissions to view
                 raise PermissionError(_("Can't view %(link)s:", link=link))
+            href = elt.attrib.get('href')
         else:
             href = req.href(link.rstrip(':'))
         req.redirect(href)
@@ -71,6 +77,9 @@ class InterTracDispatcher(Component):
                 intertrac[attribute] = value
             else:
                 intertracs[key] = value # alias
+        if 'trac' not in intertracs:
+            intertracs['trac'] = {'title': _('The Trac Project'),
+                                  'url': 'http://trac.edgewall.org'}
 
         def generate_prefix(prefix):
             intertrac = intertracs[prefix]
