@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright (C) 2005-2009 Edgewall Software
-# Copyright (C) 2005-2006 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,24 +12,22 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Christian Boos <cboos@neuf.fr>
+# Author: Christian Boos <cboos@edgewall.org>
 
 import re
-try:
-    import threading
-except ImportError:
-    import dummy_threading as threading
 
 from genshi.builder import tag
 
+from trac.cache import cached
 from trac.core import *
-from trac.wiki.formatter import Formatter
-from trac.wiki.parser import WikiParser
+from trac.util.translation import _
 from trac.wiki.api import IWikiChangeListener, IWikiMacroProvider
+from trac.wiki.parser import WikiParser
+from trac.wiki.formatter import split_url_into_path_query_fragment
 
 
 class InterWikiMap(Component):
-    """Implements support for InterWiki maps."""
+    """InterWiki map manager."""
 
     implements(IWikiChangeListener, IWikiMacroProvider)
 
@@ -37,18 +35,8 @@ class InterWikiMap(Component):
     _interwiki_re = re.compile(r"(%s)[ \t]+([^ \t]+)(?:[ \t]+#(.*))?" %
                                WikiParser.LINK_SCHEME, re.UNICODE)
     _argspec_re = re.compile(r"\$\d")
-    _interwiki_map = None
 
-    def __init__(self):
-        self._interwiki_lock = threading.RLock()
-
-    def reset(self):
-        self._interwiki_map = None
-        self.config.touch()
-        # This dictionary maps upper-cased namespaces
-        # to (namespace, prefix, title) values;
-
-    # The component itself behaves as a map
+    # The component itself behaves as a read-only map
 
     def __contains__(self, ns):
         return ns.upper() in self.interwiki_map
@@ -56,9 +44,6 @@ class InterWikiMap(Component):
     def __getitem__(self, ns):
         return self.interwiki_map[ns.upper()]
 
-    def __setitem__(self, ns, value):
-        self.interwiki_map[ns.upper()] = value
-
     def keys(self):
         return self.interwiki_map.keys()
 
@@ -85,59 +70,66 @@ class InterWikiMap(Component):
         ns, url, title = self[ns]
         maxargnum = max([0]+[int(a[1:]) for a in
                              re.findall(InterWikiMap._argspec_re, url)])
+        target, query, fragment = split_url_into_path_query_fragment(target)
         if maxargnum > 0:
             args = target.split(':', (maxargnum - 1))
         else:
             args = [target]
-        expanded_url = self._expand_or_append(url, args)
+        url = self._expand_or_append(url, args)
+        ntarget, nquery, nfragment = split_url_into_path_query_fragment(url)
+        if query and nquery:
+            nquery = '%s&%s' % (nquery, query[1:]) 
+        else:
+            nquery = nquery or query
+        nfragment = fragment or nfragment # user provided takes precedence
+        expanded_url = ntarget + nquery + nfragment
         expanded_title = self._expand(title, args)
         if expanded_title == title:
-            expanded_title = target+' in '+title
+            expanded_title = _("%(target)s in %(name)s",
+                               target=target, name=title)
         return expanded_url, expanded_title
 
     # IWikiChangeListener methods
 
     def wiki_page_added(self, page):
-        pass
+        if page.name == InterWikiMap._page_name:
+            del self.interwiki_map
 
     def wiki_page_changed(self, page, version, t, comment, author, ipnr):
         if page.name == InterWikiMap._page_name:
-            self.reset()
+            del self.interwiki_map
 
     def wiki_page_deleted(self, page):
         if page.name == InterWikiMap._page_name:
-            self.reset()
+            del self.interwiki_map
 
     def wiki_page_version_deleted(self, page):
         if page.name == InterWikiMap._page_name:
-            self.reset()
+            del self.interwiki_map
 
-    def _get_interwiki_map(self):
+    @cached
+    def interwiki_map(self, db):
+        """Map from upper-cased namespaces to (namespace, prefix, title) 
+        values.
+        """
         from trac.wiki.model import WikiPage
-        if self._interwiki_map is None:
-            self._interwiki_lock.acquire()
-            try:
-                if self._interwiki_map is None:
-                    self._interwiki_map = {}
-                    content = WikiPage(self.env, InterWikiMap._page_name).text
+        map = {}
+        content = WikiPage(self.env, InterWikiMap._page_name, db=db).text
+        in_map = False
+        for line in content.split('\n'):
+            if in_map:
+                if line.startswith('----'):
                     in_map = False
-                    for line in content.split('\n'):
-                        if in_map:
-                            if line.startswith('----'):
-                                in_map = False
-                            else:
-                                m = re.match(InterWikiMap._interwiki_re, line)
-                                if m:
-                                    prefix, url, title = m.groups()
-                                    url = url.strip()
-                                    title = title and title.strip() or prefix
-                                    self[prefix] = (prefix, url, title)
-                        elif line.startswith('----'):
-                            in_map = True
-            finally:
-                self._interwiki_lock.release()
-        return self._interwiki_map
-    interwiki_map = property(_get_interwiki_map)
+                else:
+                    m = re.match(InterWikiMap._interwiki_re, line)
+                    if m:
+                        prefix, url, title = m.groups()
+                        url = url.strip()
+                        title = title and title.strip() or prefix
+                        map[prefix.upper()] = (prefix, url, title)
+            elif line.startswith('----'):
+                in_map = True
+        return map
 
     # IWikiMacroProvider methods
 
@@ -148,7 +140,6 @@ class InterWikiMap(Component):
         return "Provide a description list for the known InterWiki prefixes."
 
     def expand_macro(self, formatter, name, content):
-        from trac.util import sorted
         interwikis = []
         for k in sorted(self.keys()):
             prefix, url, title = self[k]
