@@ -1,7 +1,8 @@
 import doctest
 import unittest
 
-from trac.wiki import api
+import trac.wiki.api
+import trac.wiki.formatter
 from trac.wiki.tests import formatter, macros, model, wikisyntax
 from trac.wiki.tests.functional import functionalSuite
 
@@ -12,7 +13,8 @@ def suite():
     suite.addTest(macros.suite())
     suite.addTest(model.suite())
     suite.addTest(wikisyntax.suite())
-    suite.addTest(doctest.DocTestSuite(api))
+    suite.addTest(doctest.DocTestSuite(trac.wiki.api))
+    suite.addTest(doctest.DocTestSuite(trac.wiki.formatter))
     return suite
 
 if __name__ == '__main__':
